use actix_web::{
	get, post, web,
	HttpResponse,
	cookie::Cookie
};
use actix_files::NamedFile;

use crate::api::types::*;
use crate::lib::users::{User, Account};
use crate::ThreadData;

use handlebars::to_json;

use maplit::btreemap;

#[post("/account/auth")]
pub async fn auth_html(
	tdata: web::Data<crate::ThreadData<'_>>,
	data: web::Data<AppState>,
	form: web::Form<CryptoUserForm>
) -> HttpResponse {
	let user = data.user_manager.lock().unwrap().get_user_by_uname(form.username.clone());
	if let Some(u) = user {
		match u.verify(form.password.clone()) {
			Ok(a) => if a {
				let mut response = HttpResponse::Ok()
					.body(tdata.handlebars.render("auth", &btreemap!("user" => u.clone())).unwrap());
				response.add_cookie(
					&Cookie::build("auth", format!("{}&{}", u.user_id, u.secret))
						.domain("localhost")
						//.domain("localhost")
						.path("/")
						.same_site(actix_web::cookie::SameSite::Strict)
						//.secure(true)
						.finish()
				).unwrap();
				response
			} else {
				HttpResponse::Forbidden().body("Forbidden: Could not authenticate user")
			}
			Err(e) => HttpResponse::BadRequest().body(format!("Bad Request: {}", e))
		}
	} else {
		HttpResponse::BadRequest().body("Bad Request: User does not exist")
	}
}

#[get("/account/login/")]
pub async fn login() -> actix_web::Result<NamedFile> {
	Ok(NamedFile::open("static/account_login.html")?)
}

#[post("/account/new")]
pub async fn sign_up_result(
	data: web::Data<AppState>,
	form: web::Form<CryptoUserForm>,
	tdata: web::Data<ThreadData<'_>>
) -> HttpResponse {
	println!("new user");
	let mut um = data.user_manager.lock().unwrap();
	let user = {
		if let Some(_a) = um.get_user_by_uname(form.username.clone()) {
			return HttpResponse::Conflict().body("Conflict: user with that username already exists");
		} else {
			let new_id = um.assign_id();
			User::new(new_id, &form.username).encrypt(form.password.clone())
		}
	};
	um.add_user(user.clone());
	HttpResponse::Created().body(tdata.handlebars.render("new_user_redirect", &btreemap! {
		"user" => to_json(user.username)
	}).unwrap())
}

#[get("/account/new/")]
pub async fn sign_up() -> actix_web::Result<NamedFile> {
	Ok(NamedFile::open("static/new_account.html")?)
}
