pub mod accounts;
pub mod boards;

use std::time::Duration;

trait TimeDerive {
	fn derive_time_since(&self, time: Duration) -> Option<(f64, String)>;
}

impl TimeDerive for Duration {
	fn derive_time_since(&self, time: Duration) -> Option<(f64, String)> {
		time.checked_sub(*self).map(|a| {
			match a.as_secs() {
				t if t > 604800 => (t as f64/60.0/60.0/24.0/7.0, "weeks".to_string()),
				t if t > 86400 => (t as f64/60.0/60.0/24.0, "days".to_string()),
				t if t > 3600 => (t as f64/60.0/60.0, "hours".to_string()),
				t if t > 120 => (t as f64/60.0, "mins".to_string()),
				t => (t as f64, "secs".to_string())
			}
		})
	}
}
